part of masamune.flutter;

/// Class that manages values
/// Please use in conjunction with UIWidget.
/// Usually used to get the value using [get()] and [text()]
class UIValue {
  _UIWidgetState _state;
  Map<String, Observer> _data = MapPool.get();
  UIValue._( _UIWidgetState state ) : this._state = state;
  /// Get the [UIValue] closest to the build context
  /// 
  /// [context]: Build context
  static UIValue of( BuildContext context ){
    if( context == null ) return null;
    _UIWidgetState widget = _ContextMap.get(context);
    if( widget == null ){
      widget = context.findAncestorStateOfType<_UIWidgetState>();
      if( widget != null ) _ContextMap.register(context, widget);
    }
    return widget?._value;
  }
  /// Get object from pathmap in conjunction with UIWidget
  /// You can get any IPath object other than String
  /// Please use it for update processing in the widget
  /// 
  /// [path]: The path to get
  /// [defaultValue]: Value if none
  /// [filter]: Filter values.
  /// [listen]: True to listen for values
  T get<T extends Object>( String path, {T defaultValue, T filter( T value ), bool listen = false} ){
    if( isEmpty(path) ) return filter != null ? filter( PathMap.get<T>(path) ?? defaultValue ) : ( PathMap.get<T>(path) ?? defaultValue );
    if( listen ) this._addPath(path);
    return  filter != null ? filter( PathMap.get<T>(path) ?? defaultValue ) : ( PathMap.get<T>(path) ?? defaultValue );
  }
  /// Get text from pathmap in conjunction with UIWidget
  /// Displays anything as long as Path is Unit and can be converted to text
  /// If not, [defaultValue] will be displayed
  /// 
  /// [path]: The path to get
  /// [defaultValue]: Value if none
  /// [filter]: Filter values.
  /// [listen]: True to listen for values
  String text( String path, {String defaultValue = Const.empty, String filter( String value ), bool listen = true } ) => filter != null ? filter( this.get<dynamic>( path, listen: listen )?.toString() ?? defaultValue ) : ( this.get<dynamic>( path, listen: true )?.toString() ?? defaultValue );
  /// Get localized text from pathmap in conjunction with UIWidget
  /// Displays anything as long as Path is Unit and can be converted to text
  /// If not, [defaultValue] will be displayed
  /// 
  /// [path]: The path to get
  /// [defaultValue]: Value if none
  /// [filter]: Filter values.
  /// [listen]: True to listen for values
  String localized( String path, {String defaultValue = Const.empty, String filter( String value ), bool listen = true } ) => filter != null ? filter( Localize.get( this.get<dynamic>( path, listen: listen )?.toString() ?? defaultValue ) ) : ( Localize.get( this.get<dynamic>( path, listen: true )?.toString() ?? defaultValue ) );
  /// Get number from pathmap in conjunction with UIWidget
  /// Displays anything as long as Path is Unit and can be converted to double
  /// If not, [defaultValue] will be displayed
  /// 
  /// [path]: The path to get
  /// [defaultValue]: Value if none
  /// [filter]: Filter values.
  /// [listen]: True to listen for values
  double number( String path, {double defaultValue = 0, double filter( double value ), bool listen = true } ) {
    dynamic tmp = this.get<dynamic>( path, listen: listen );
    if( tmp is double ) return filter != null ? filter( tmp ) : tmp;
    if( tmp is int ) return filter != null ? filter( tmp as double ) : tmp as int;
    return filter != null ? filter( defaultValue ) : defaultValue;
  }
  /// Get the action callback from pathmap in conjunction with UIWidget.
  /// If it does not exist, the action that does nothing is called.
  /// 
  /// [path]: The path to get
  /// [listen]: True to listen for values
  /// [defaultAction]: Default callback when no value exists in path
  VoidAction action( String path, { bool listen = false, VoidAction defaultAction } ){
    return this.get<VoidAction>( path, listen: listen ) ?? defaultAction ?? (){};
  }
  void _addPath( String path ){
    if( isEmpty(path) || this._data.containsKey(path) ) return;
    Observer observer = Observer();
    PathListener.listen(path, this._notifyUpdate, observer:observer );
    this._data[path] = observer;
  }
  void _notifyUpdate( dynamic object ) {
     this._state?._notifyUpdate();
  }
  void _dispose(){
    for( Observer tmp in this._data.values ) tmp.dispose();
    this._data.release();
    this._state = null;
  }
}